package SolutionsExercise2 "Example solutions of exercise 2 of Advanced Modelica Tutorial at Modelica'2002"
  import Modelica.SIunits;

model testDrive
  import SI = Modelica.SIunits;
  extends Modelica.Icons.Example;
  parameter Real angleBegDeg=0 "start angle in degree";
  parameter Real angleEndDeg=360 "end angle in degree";

  SI.Angle angleError=drive.angleError;
  SI.AngularVelocity speedError=pathPlanning.integratorVel.outPort.signal[1] -
      drive.speed.w;
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]), Window(
      x=0.06,
      y=0.05,
      width=0.79,
      height=0.75));
  ServoLib1.Drive drive annotation (extent=[0, 0; 20, 20]);
  Modelica.Mechanics.Rotational.Inertia load(J=50) annotation (extent=[40, 0;
        60, 20]);
  ServoLib1.PathPlanning pathPlanning(
    naxis=1,
    angleBegDeg={angleBegDeg},
    angleEndDeg={angleEndDeg}) annotation (extent=[-40, 0; -20, 20]);
equation
  connect(pathPlanning.refAngles, drive.refAngle) annotation (points=[
        -20, 10; 0, 10], style(color=3));
  connect(drive.flange, load.flange_a) annotation (points=[22, 10; 40, 10],
      style(color=0));
end testDrive;


model robot
  import SI = Modelica.SIunits;
  extends Modelica.Icons.Example;
  parameter Real angleBegDeg[2]={0,0} "start angles in degree";
  parameter Real angleEndDeg[2]={90,90} "end angles in degree";

  parameter Real kp=10 "gain of position controllers";
  parameter Real ks=0.6 "gain of speed controllers";
  parameter Real Ts=0.08 "time constant of PI controller";

  SI.Angle errAngle[2]={drive1.feedback1.outPort.signal[1],drive2.feedback1.
      outPort.signal[1]} "angle error";
  SI.AngularVelocity errSpeed[2]=pathPlanning.integratorVel.outPort.signal - {
      drive1.speed.w,drive2.speed.w} "speed error";

public
  ServoLib1.PathPlanning pathPlanning(angleBegDeg=angleBegDeg, angleEndDeg=
        angleEndDeg) annotation (extent=[-80, 0; -60, 20]);
  ServoLib1.Drive drive1(
    kp=kp,
    ks=ks,
    Ts=Ts) annotation (extent=[0, -20; 20, 0]);
  ServoLib1.RobotStructure robotStructure annotation (extent=[50, -10; 90, 30])
    ;
  ServoLib1.Drive drive2(
    kp=kp,
    ks=ks,
    Ts=Ts) annotation (extent=[0, 20; 20, 40]);
  ModelicaAdditions.Blocks.Multiplexer.DeMultiplex2 DeMux annotation (extent=[
        -40, 20; -20, 0]);
equation
  connect(pathPlanning.refAngles, DeMux.inPort) annotation (points=[
        -60, 10; -42, 10], style(color=3));
  connect(drive2.flange, robotStructure.axis2) annotation (points=[20, 30; 30,
        30; 30, 18; 49, 18], style(color=0, thickness=2));
  connect(drive1.flange, robotStructure.axis1) annotation (points=[22, -10; 30
        , -10; 30, -6; 49, -6], style(color=0, thickness=2));
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[1, 1],
      component=[20, 20]), Window(
      x=0.06,
      y=0.03,
      width=0.89,
      height=0.83));
  connect(DeMux.outPort2, drive2.refAngle) annotation (points=[-19, 16; -10, 16
        ; -10, 30; -1, 30], style(color=3));
  connect(DeMux.outPort1, drive1.refAngle) annotation (points=[-18, 4; -10, 4;
        -10, -10; -1, -10], style(color=3));
end robot;


model robot2
  extends robot(
    angleBegDeg={45,45},
    angleEndDeg={135,-90},
    pathPlanning(integratorPos(y0=pathPlanning.angleBeg, y(each fixed=true)),
        integratorVel(y0=zeros(2), y(each fixed=true))),
    drive1(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    drive2(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    robotStructure(axis1(phi(start=pathPlanning.angleBeg[1], fixed=false)),
        axis2(phi(start=pathPlanning.angleBeg[2], fixed=false))));

  Real x1=drive1.PI.x[1];
  Real x2=drive2.PI.x[1];
  Real w1=der(robotStructure.axis1.phi);
  Real w2=der(robotStructure.axis2.phi);
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[1, 1],
      component=[20, 20]), Window(
      x=0.04,
      y=0.06,
      width=0.76,
      height=0.87));
initial equation
  w1 = 0;
  w2 = 0;
  der(w1) = 0;
  der(w2) = 0;
  der(x1) = 0;
  der(x2) = 0;
end robot2;

model robot3
  extends robot(
    angleBegDeg={45,45},
    angleEndDeg={135,-90},
    pathPlanning(integratorPos(y0=pathPlanning.angleBeg, y(each fixed=true)),
        integratorVel(y0=zeros(2), y(each fixed=true))),
    drive1(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    drive2(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    robotStructure(axis1(phi(start=pathPlanning.angleBeg[1], fixed=true)),
        axis2(phi(start=pathPlanning.angleBeg[2], fixed=true))));

  Real w1=der(robotStructure.axis1.phi);
  Real w2=der(robotStructure.axis2.phi);
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[1, 1],
      component=[20, 20]), Window(
      x=0.01,
      y=0.02,
      width=0.76,
      height=0.83));
initial equation
  w1 = 0;
  w2 = 0;
  der(w1) = 0;
  der(w2) = 0;
end robot3;

model sampledRobot
  import SI = Modelica.SIunits;
  extends Modelica.Icons.Example;
  parameter Real angleBegDeg[2]={0,0} "start angles in degree";
  parameter Real angleEndDeg[2]={90,90} "end angles in degree";

  parameter Real kp=10 "gain of position controllers";
  parameter Real ks=0.6 "gain of speed controllers";
  parameter Real Ts=0.08 "time constant of PI controller";
  parameter Real sampleTime=0.005;

  SI.Angle errAngle[2]={drive1.feedback1.outPort.signal[1],drive2.feedback1.
      outPort.signal[1]} "angle error";
  SI.AngularVelocity errSpeed[2]=pathPlanning.integratorVel.outPort.signal - {
      drive1.speed.w,drive2.speed.w} "speed error";

public
  ServoLib1.PathPlanning pathPlanning(angleBegDeg=angleBegDeg, angleEndDeg=
        angleEndDeg) annotation (extent=[-80, 0; -60, 20]);
  ServoLib1.SampledDrive drive1(
    kp=kp,
    ks=ks,
    Ts=Ts,
    sampleTime=sampleTime) annotation (extent=[0, -20; 20, 0]);
  ServoLib1.RobotStructure robotStructure annotation (extent=[50, -10; 90, 30])
    ;
  ServoLib1.SampledDrive drive2(
    kp=kp,
    ks=ks,
    Ts=Ts,
    sampleTime=sampleTime) annotation (extent=[0, 20; 20, 40]);
  ModelicaAdditions.Blocks.Multiplexer.DeMultiplex2 DeMux annotation (extent=[
        -40, 20; -20, 0]);
equation
  connect(pathPlanning.refAngles, DeMux.inPort) annotation (points=[
        -60, 10; -42, 10], style(color=3));
  connect(drive2.flange, robotStructure.axis2) annotation (points=[20, 30; 30,
        30; 30, 18; 49, 18], style(color=0, thickness=2));
  connect(drive1.flange, robotStructure.axis1) annotation (points=[22, -10; 30
        , -10; 30, -6; 49, -6], style(color=0, thickness=2));
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[1, 1],
      component=[20, 20]), Window(
      x=0.06,
      y=0.03,
      width=0.89,
      height=0.83));
  connect(DeMux.outPort2, drive2.refAngle) annotation (points=[-19, 16; -10, 16
        ; -10, 30; -1, 30], style(color=3));
  connect(DeMux.outPort1, drive1.refAngle) annotation (points=[-18, 4; -10, 4;
        -10, -10; -1, -10], style(color=3));
end sampledRobot;

model sampledRobot3
  extends sampledRobot(
    angleBegDeg={45,45},
    angleEndDeg={135,-90},
    pathPlanning(integratorPos(y0=pathPlanning.angleBeg, y(each fixed=true)),
        integratorVel(y0=zeros(2), y(each fixed=true))),
    drive1(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    drive2(Jmotor(w(start=0, fixed=true), a(start=0, fixed=true))),
    robotStructure(axis1(phi(start=pathPlanning.angleBeg[1], fixed=true)),
        axis2(phi(start=pathPlanning.angleBeg[2], fixed=true))));

  Real w1=der(robotStructure.axis1.phi);
  Real w2=der(robotStructure.axis2.phi);
  annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[1, 1],
      component=[20, 20]), Window(
      x=0.01,
      y=0.02,
      width=0.76,
      height=0.83));
equation

initial equation
  w1 = 0;
  w2 = 0;
  der(w1) = 0;
  der(w2) = 0;
end sampledRobot3;


end SolutionsExercise2;